/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/
/*
    bufqmr.c

    This is for buffer management.

    2009-8-25	CF Lin
		new release
*/
#include "bufqmr.h"

void
bufqmr_free(BUFQMR *q)
{
	if (q)
	{
		if (q->head) buffer_free(q->head);
		FREE(q);
	}
}

BUFQMR*
bufqmr_new (int num_buf, int buf_size)
{
	BUFQMR *q;
    BUFFER *b;

	q = MALLOC (sizeof (BUFQMR));
	if (!q)
		return NULL;

	q->head=NULL;
	spin_lock_init(&q->lock_qmr);
	while (num_buf--)
	{
		b = buffer_new(buf_size);
		if (!b)
		{
			bufqmr_free(q);
			return NULL;
		}
		q->head = buffer_append(q->head, b);
	}
	return q;
}

void
bufqmr_append_buffer (BUFQMR *q, BUFFER *b)
{
	if (!q || !b)
		return;
    if (q->head)
	{
		spin_lock(&q->lock_qmr);
		q->head = buffer_append(q->head, b);
		spin_unlock(&q->lock_qmr);
	}
}

void
bufqmr_push_buffer (BUFQMR *q, BUFFER *b)
{
	if (!q || !b)
		return;
    if (q->head)
	{
		spin_lock(&q->lock_qmr);
		q->head = buffer_push(q->head, b);
		spin_unlock(&q->lock_qmr);
	}
}

BUFFER*
bufqmr_pop_buffer (BUFQMR *q)
{
    BUFFER *b = NULL;

	if (!q)
		return NULL;

	spin_lock(&q->lock_qmr);
    if (q->head)
	{
		b = q->head;
		q->head = q->head->next;
		b->next = NULL;
	}
	spin_unlock(&q->lock_qmr);
	return b;
}


